#include "k3bblankingjob.h"

#include "k3b.h"
#include "k3bglobals.h"
#include "device/k3bdevice.h"

#include <kprocess.h>
#include <kconfig.h>
#include <klocale.h>

#include <qstring.h>

#include <iostream>


K3bBlankingJob::K3bBlankingJob()
  : K3bJob()
{
  m_process = new KProcess();
  m_device = 0;
  m_mode = Fast;
  m_speed = 1;

  // connect to the cdrecord slots
  connect( m_process, SIGNAL(processExited(KProcess*)),
	   this, SLOT(slotCdrecordFinished()) );
  connect( m_process, SIGNAL(receivedStderr(KProcess*, char*, int)),
	   this, SLOT(slotParseCdrecordOutput(KProcess*, char*, int)) );
  connect( m_process, SIGNAL(receivedStdout(KProcess*, char*, int)),
	   this, SLOT(slotParseCdrecordOutput(KProcess*, char*, int)) );
}


K3bBlankingJob::~K3bBlankingJob()
{
  delete m_process;
}


void K3bBlankingJob::setDevice( K3bDevice* dev )
{
  m_device = dev;
}


void K3bBlankingJob::start()
{
  if( m_process->isRunning() )
    return;

  if( m_device == 0 )
    return;

  m_process->clearArguments();
  KConfig* c = kapp->config();
  c->setGroup( "External Programs" );
  *m_process << c->readEntry( "cdrecord path" );
  *m_process << QString("dev=%1").arg( m_device->devicename() );
  *m_process << QString("speed=%1").arg(m_speed);
  if( m_force )
    *m_process << "-force";
  if( k3bMain()->eject() )
    *m_process << "-eject";

  QString mode;
  switch( m_mode ) {
  case Fast:
    mode = "fast";
    break;
  case Complete:
    mode = "all";
    break;
  case Track:
    mode = "track";
    break;
  case Unclose:
    mode = "unclose";
    break;
  case Session:
    mode = "session";
    break;
  }

  *m_process << "blank=" + mode;


  // debugging output
  cout << "***** cdrecord parameters:\n";
  QStrList* _args = m_process->args();
  QStrListIterator _it(*_args);
  while( _it ) {
    cout << *_it << " ";
    ++_it;
  }
  cout << endl << flush;



  // now we can start the process
  if( !m_process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) )
    {
      // something went wrong when starting the program
      // it "should" be the executable
      qDebug("(K3bBlankingJob) could not start cdrecord");
      emit infoMessage( i18n("Could not start cdrecord!"), K3bJob::ERROR );
      emit finished( false );
    }
  else
    {
      emit infoMessage( i18n("Start blanking disc at speed %1...").arg(m_speed), K3bJob::STATUS );
    }
}


void K3bBlankingJob::cancel()
{
  if( m_process->isRunning() ) {
    m_process->kill();

    // we need to unlock the writer because cdrecord locked it while writing
    bool block = m_device->block( false );
    if( !block )
      emit infoMessage( i18n("Could not unlock cd drive."), K3bJob::ERROR );
    //    else if( k3bMain()->eject() )
    // m_device->eject();

    emit canceled();
    emit finished( false );
  }
}


void K3bBlankingJob::slotParseCdrecordOutput( KProcess*, char* data, int len )
{
  qDebug( QString::fromLatin1( data, len ) );
}


void K3bBlankingJob::slotCdrecordFinished()
{
  if( m_process->normalExit() )
    {
      // TODO: check the process' exitStatus()
      switch( m_process->exitStatus() )
	{
	case 0:
	  emit infoMessage( i18n("Process successfully finished"), K3bJob::STATUS );
	  emit finished( true );
	  break;
				
	default:
	  // no recording device and also other errors!! :-(
	  emit infoMessage( i18n("Cdrecord returned some error!"), K3bJob::ERROR );
	  emit infoMessage( i18n("Sorry, no error handling yet! :-(("), K3bJob::ERROR );
	  emit finished( false );
	  break;
	}
    }
  else
    {
      emit infoMessage( i18n("Cdrecord did not exit cleanly."), K3bJob::ERROR );
      emit finished( false );
    }
}


bool K3bBlankingJob::active() const
{
  return m_process->isRunning();
}


#include "k3bblankingjob.moc"
